# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.cluster.models.worm_specific_target_params import WormSpecificTargetParams
from typing import Set
from typing_extensions import Self

class CommonArchivalAzureExternalTargetParams(BaseModel):
    """
    Specifies the common parameters which are specific to Azure related External Targets of archival purpose type.
    """ # noqa: E501
    client_id: Optional[StrictStr] = Field(default=None, description="Specifies the client id of the managed identity assigned to the cluster This is used only for clusters running as Azure VMs where authentication is done using AD.", alias="clientId")
    container_name: Optional[StrictStr] = Field(description="Specifies the container name of the external target.", alias="containerName")
    region: Optional[StrictStr] = Field(default=None, description="Specifies region of the External Target. This is only populated for FortKnox vaults.")
    storage_access_key: Optional[StrictStr] = Field(default=None, description="Specifies the storage access key of the external target.", alias="storageAccessKey")
    storage_account_name: Optional[StrictStr] = Field(description="Specifies the storage account name of the external target.", alias="storageAccountName")
    is_forever_incremental_archival_enabled: Optional[StrictBool] = Field(default=None, description="Specifies if Forever Incremental Archival setting is enabled or not.", alias="isForeverIncrementalArchivalEnabled")
    is_incremental_archival_enabled: Optional[StrictBool] = Field(default=None, description="Specifies if Incremental Archival setting is enabled or not.", alias="isIncrementalArchivalEnabled")
    is_worm_enabled: Optional[StrictBool] = Field(default=None, description="Specifies whether write once read many (WORM) protection is enabled for the Azure container or not.", alias="isWormEnabled")
    source_side_deduplication: Optional[StrictBool] = Field(default=None, description="Specifies the Source Side Deduplication setting for the Azure external target", alias="sourceSideDeduplication")
    storage_class: Optional[StrictStr] = Field(description="Specifies the Azure External Target storage class.", alias="storageClass")
    worm_specific_target_params: Optional[WormSpecificTargetParams] = Field(default=None, alias="wormSpecificTargetParams")
    __properties: ClassVar[List[str]] = ["clientId", "containerName", "region", "storageAccessKey", "storageAccountName", "isForeverIncrementalArchivalEnabled", "isIncrementalArchivalEnabled", "isWormEnabled", "sourceSideDeduplication", "storageClass", "wormSpecificTargetParams"]

    @field_validator('storage_class')
    def storage_class_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['AzureArchiveBlob', 'AzureCoolBlob', 'AzureHotBlob']):
            raise ValueError("must be one of enum values ('AzureArchiveBlob', 'AzureCoolBlob', 'AzureHotBlob')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CommonArchivalAzureExternalTargetParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of worm_specific_target_params
        if self.worm_specific_target_params:
            _dict['wormSpecificTargetParams'] = self.worm_specific_target_params.to_dict()
        # set to None if client_id (nullable) is None
        # and model_fields_set contains the field
        if self.client_id is None and "client_id" in self.model_fields_set:
            _dict['clientId'] = None

        # set to None if container_name (nullable) is None
        # and model_fields_set contains the field
        if self.container_name is None and "container_name" in self.model_fields_set:
            _dict['containerName'] = None

        # set to None if region (nullable) is None
        # and model_fields_set contains the field
        if self.region is None and "region" in self.model_fields_set:
            _dict['region'] = None

        # set to None if storage_access_key (nullable) is None
        # and model_fields_set contains the field
        if self.storage_access_key is None and "storage_access_key" in self.model_fields_set:
            _dict['storageAccessKey'] = None

        # set to None if storage_account_name (nullable) is None
        # and model_fields_set contains the field
        if self.storage_account_name is None and "storage_account_name" in self.model_fields_set:
            _dict['storageAccountName'] = None

        # set to None if is_forever_incremental_archival_enabled (nullable) is None
        # and model_fields_set contains the field
        if self.is_forever_incremental_archival_enabled is None and "is_forever_incremental_archival_enabled" in self.model_fields_set:
            _dict['isForeverIncrementalArchivalEnabled'] = None

        # set to None if is_incremental_archival_enabled (nullable) is None
        # and model_fields_set contains the field
        if self.is_incremental_archival_enabled is None and "is_incremental_archival_enabled" in self.model_fields_set:
            _dict['isIncrementalArchivalEnabled'] = None

        # set to None if is_worm_enabled (nullable) is None
        # and model_fields_set contains the field
        if self.is_worm_enabled is None and "is_worm_enabled" in self.model_fields_set:
            _dict['isWormEnabled'] = None

        # set to None if source_side_deduplication (nullable) is None
        # and model_fields_set contains the field
        if self.source_side_deduplication is None and "source_side_deduplication" in self.model_fields_set:
            _dict['sourceSideDeduplication'] = None

        # set to None if storage_class (nullable) is None
        # and model_fields_set contains the field
        if self.storage_class is None and "storage_class" in self.model_fields_set:
            _dict['storageClass'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CommonArchivalAzureExternalTargetParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "clientId": obj.get("clientId"),
            "containerName": obj.get("containerName"),
            "region": obj.get("region"),
            "storageAccessKey": obj.get("storageAccessKey"),
            "storageAccountName": obj.get("storageAccountName"),
            "isForeverIncrementalArchivalEnabled": obj.get("isForeverIncrementalArchivalEnabled"),
            "isIncrementalArchivalEnabled": obj.get("isIncrementalArchivalEnabled"),
            "isWormEnabled": obj.get("isWormEnabled"),
            "sourceSideDeduplication": obj.get("sourceSideDeduplication"),
            "storageClass": obj.get("storageClass"),
            "wormSpecificTargetParams": WormSpecificTargetParams.from_dict(obj["wormSpecificTargetParams"]) if obj.get("wormSpecificTargetParams") is not None else None
        })
        return _obj


