# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.cluster.models.local_group_params import LocalGroupParams
from cohesity_sdk.cluster.models.smb_principal import SMBPrincipal
from typing import Set
from typing_extensions import Self

class GroupParams(BaseModel):
    """
    Specifies a Group.
    """ # noqa: E501
    description: Optional[StrictStr] = Field(default=None, description="Specifies the description of the group.")
    domain: StrictStr = Field(description="Specifies the domain of the group. For active directories, this is the fully qualified domain name (FQDN). It is 'LOCAL' for local groups on the Cohesity Cluster. A group is uniquely identified by combination of the name and the domain.")
    local_group_params: Optional[LocalGroupParams] = Field(default=None, alias="localGroupParams")
    name: StrictStr = Field(description="Specifies the name of the group.")
    restricted: Optional[StrictBool] = Field(default=None, description="Specifies whether the Group is restricted. A restricted group can only view & manage the objects it has permissions to.")
    roles: Optional[List[StrictStr]] = Field(default=None, description="Specifies the Cohesity roles to associate with the group. The Cohesity roles determine privileges on the Cohesity Cluster for this group.")
    tenant_ids: Optional[List[StrictStr]] = Field(default=None, description="Specifies a list of tenant ids who can access this group.", alias="tenantIds")
    created_time_msecs: Optional[StrictInt] = Field(default=None, description="Specifies the epoch time in milliseconds when the group was created.", alias="createdTimeMsecs")
    last_updated_time_msecs: Optional[StrictInt] = Field(default=None, description="Specifies the epoch time in milliseconds when the group was last modified.", alias="lastUpdatedTimeMsecs")
    sid: Optional[StrictStr] = Field(default=None, description="Specifies the sid of the Group.")
    smb_principals: Optional[List[SMBPrincipal]] = Field(default=None, description="Specifies the SMB principals.", alias="smbPrincipals")
    __properties: ClassVar[List[str]] = ["description", "domain", "localGroupParams", "name", "restricted", "roles", "tenantIds", "createdTimeMsecs", "lastUpdatedTimeMsecs", "sid", "smbPrincipals"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GroupParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "created_time_msecs",
            "last_updated_time_msecs",
            "sid",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of local_group_params
        if self.local_group_params:
            _dict['localGroupParams'] = self.local_group_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in smb_principals (list)
        _items = []
        if self.smb_principals:
            for _item_smb_principals in self.smb_principals:
                if _item_smb_principals:
                    _items.append(_item_smb_principals.to_dict())
            _dict['smbPrincipals'] = _items
        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if restricted (nullable) is None
        # and model_fields_set contains the field
        if self.restricted is None and "restricted" in self.model_fields_set:
            _dict['restricted'] = None

        # set to None if created_time_msecs (nullable) is None
        # and model_fields_set contains the field
        if self.created_time_msecs is None and "created_time_msecs" in self.model_fields_set:
            _dict['createdTimeMsecs'] = None

        # set to None if last_updated_time_msecs (nullable) is None
        # and model_fields_set contains the field
        if self.last_updated_time_msecs is None and "last_updated_time_msecs" in self.model_fields_set:
            _dict['lastUpdatedTimeMsecs'] = None

        # set to None if sid (nullable) is None
        # and model_fields_set contains the field
        if self.sid is None and "sid" in self.model_fields_set:
            _dict['sid'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GroupParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "description": obj.get("description"),
            "domain": obj.get("domain"),
            "localGroupParams": LocalGroupParams.from_dict(obj["localGroupParams"]) if obj.get("localGroupParams") is not None else None,
            "name": obj.get("name"),
            "restricted": obj.get("restricted"),
            "roles": obj.get("roles"),
            "tenantIds": obj.get("tenantIds"),
            "createdTimeMsecs": obj.get("createdTimeMsecs"),
            "lastUpdatedTimeMsecs": obj.get("lastUpdatedTimeMsecs"),
            "sid": obj.get("sid"),
            "smbPrincipals": [SMBPrincipal.from_dict(_item) for _item in obj["smbPrincipals"]] if obj.get("smbPrincipals") is not None else None
        })
        return _obj


