# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.cluster.models.key_value_pair import KeyValuePair
from cohesity_sdk.cluster.models.recover_cassandra_snapshot_params import RecoverCassandraSnapshotParams
from typing import Set
from typing_extensions import Self

class RecoverCassandraParams(BaseModel):
    """
    Specifies the parameters to recover Cassandra objects.
    """ # noqa: E501
    advanced_configs: Optional[List[KeyValuePair]] = Field(default=None, description="Specifies the advanced configuration for a recovery job.", alias="advancedConfigs")
    bandwidth_mbps: Optional[StrictInt] = Field(default=None, description="Specifies the maximum network bandwidth that each concurrent IO Stream can use for exchanging data with the cluster.", alias="bandwidthMBPS")
    concurrency: Optional[StrictInt] = Field(default=None, description="Specifies the maximum number of concurrent IO Streams that will be created to exchange data with the cluster.")
    overwrite: Optional[StrictBool] = Field(default=None, description="Set to true to overwrite an existing object at the destination. If set to false, and the same object exists at the destination, then recovery will fail for that object.")
    recover_to: Optional[StrictInt] = Field(default=None, description="Specifies the 'Source Registration ID' of the source where the objects are to be recovered. If this is not specified, the recovery job will recover to the original location.", alias="recoverTo")
    warnings: Optional[List[StrictStr]] = Field(default=None, description="This field will hold the warnings in cases where the job status is SucceededWithWarnings.")
    is_live_table_restore: Optional[StrictBool] = Field(default=None, description="Specifies whether the current recovery operation is a live table restore operation.", alias="isLiveTableRestore")
    is_system_keyspace_restore: Optional[StrictBool] = Field(default=None, description="Specifies whether the current recovery operation is a system keyspace restore operation.", alias="isSystemKeyspaceRestore")
    log_restore_directory: Optional[StrictStr] = Field(default=None, description="Specifies the directory for restoring the logs.", alias="logRestoreDirectory")
    recover_privileges: Optional[StrictBool] = Field(default=None, description="Specifies whether recover/skip roles and permissions.", alias="recoverPrivileges")
    restart_at_usecs: Optional[StrictInt] = Field(default=None, description="Specifies the time in Unix epoch timestamp in microseconds at which the Cassandra services are to be restarted.", alias="restartAtUsecs")
    restart_command: Optional[StrictStr] = Field(default=None, description="Specifies the command to restart Cassandra services after the point in time recovery.", alias="restartCommand")
    restart_immediately: Optional[StrictBool] = Field(default=None, description="Specifies whether to restart Cassandra services immediately after the point in time recovery.", alias="restartImmediately")
    restart_services: Optional[StrictBool] = Field(default=None, description="Specifies whether to restart Cassandra services after the point in time recovery.", alias="restartServices")
    restart_services_task_id: Optional[StrictInt] = Field(default=None, description="Specifies the Id of the task required to restart Cassandra services.", alias="restartServicesTaskId")
    selected_data_centers: Optional[List[StrictStr]] = Field(default=None, description="Selected Data centers for this cluster.", alias="selectedDataCenters")
    snapshots: Optional[List[RecoverCassandraSnapshotParams]] = Field(description="Specifies the local snapshot ids and other details of the Objects to be recovered.")
    staging_directory_list: Optional[List[StrictStr]] = Field(default=None, description="Specifies the directory on the primary to copy the files which are to be uploaded using destination sstableloader.", alias="stagingDirectoryList")
    suffix: Optional[StrictStr] = Field(default=None, description="A suffix that is to be applied to all recovered objects.")
    __properties: ClassVar[List[str]] = ["advancedConfigs", "bandwidthMBPS", "concurrency", "overwrite", "recoverTo", "warnings", "isLiveTableRestore", "isSystemKeyspaceRestore", "logRestoreDirectory", "recoverPrivileges", "restartAtUsecs", "restartCommand", "restartImmediately", "restartServices", "restartServicesTaskId", "selectedDataCenters", "snapshots", "stagingDirectoryList", "suffix"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RecoverCassandraParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "warnings",
            "restart_services_task_id",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in advanced_configs (list)
        _items = []
        if self.advanced_configs:
            for _item_advanced_configs in self.advanced_configs:
                if _item_advanced_configs:
                    _items.append(_item_advanced_configs.to_dict())
            _dict['advancedConfigs'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in snapshots (list)
        _items = []
        if self.snapshots:
            for _item_snapshots in self.snapshots:
                if _item_snapshots:
                    _items.append(_item_snapshots.to_dict())
            _dict['snapshots'] = _items
        # set to None if advanced_configs (nullable) is None
        # and model_fields_set contains the field
        if self.advanced_configs is None and "advanced_configs" in self.model_fields_set:
            _dict['advancedConfigs'] = None

        # set to None if bandwidth_mbps (nullable) is None
        # and model_fields_set contains the field
        if self.bandwidth_mbps is None and "bandwidth_mbps" in self.model_fields_set:
            _dict['bandwidthMBPS'] = None

        # set to None if concurrency (nullable) is None
        # and model_fields_set contains the field
        if self.concurrency is None and "concurrency" in self.model_fields_set:
            _dict['concurrency'] = None

        # set to None if overwrite (nullable) is None
        # and model_fields_set contains the field
        if self.overwrite is None and "overwrite" in self.model_fields_set:
            _dict['overwrite'] = None

        # set to None if recover_to (nullable) is None
        # and model_fields_set contains the field
        if self.recover_to is None and "recover_to" in self.model_fields_set:
            _dict['recoverTo'] = None

        # set to None if warnings (nullable) is None
        # and model_fields_set contains the field
        if self.warnings is None and "warnings" in self.model_fields_set:
            _dict['warnings'] = None

        # set to None if is_live_table_restore (nullable) is None
        # and model_fields_set contains the field
        if self.is_live_table_restore is None and "is_live_table_restore" in self.model_fields_set:
            _dict['isLiveTableRestore'] = None

        # set to None if is_system_keyspace_restore (nullable) is None
        # and model_fields_set contains the field
        if self.is_system_keyspace_restore is None and "is_system_keyspace_restore" in self.model_fields_set:
            _dict['isSystemKeyspaceRestore'] = None

        # set to None if log_restore_directory (nullable) is None
        # and model_fields_set contains the field
        if self.log_restore_directory is None and "log_restore_directory" in self.model_fields_set:
            _dict['logRestoreDirectory'] = None

        # set to None if recover_privileges (nullable) is None
        # and model_fields_set contains the field
        if self.recover_privileges is None and "recover_privileges" in self.model_fields_set:
            _dict['recoverPrivileges'] = None

        # set to None if restart_at_usecs (nullable) is None
        # and model_fields_set contains the field
        if self.restart_at_usecs is None and "restart_at_usecs" in self.model_fields_set:
            _dict['restartAtUsecs'] = None

        # set to None if restart_command (nullable) is None
        # and model_fields_set contains the field
        if self.restart_command is None and "restart_command" in self.model_fields_set:
            _dict['restartCommand'] = None

        # set to None if restart_immediately (nullable) is None
        # and model_fields_set contains the field
        if self.restart_immediately is None and "restart_immediately" in self.model_fields_set:
            _dict['restartImmediately'] = None

        # set to None if restart_services (nullable) is None
        # and model_fields_set contains the field
        if self.restart_services is None and "restart_services" in self.model_fields_set:
            _dict['restartServices'] = None

        # set to None if restart_services_task_id (nullable) is None
        # and model_fields_set contains the field
        if self.restart_services_task_id is None and "restart_services_task_id" in self.model_fields_set:
            _dict['restartServicesTaskId'] = None

        # set to None if snapshots (nullable) is None
        # and model_fields_set contains the field
        if self.snapshots is None and "snapshots" in self.model_fields_set:
            _dict['snapshots'] = None

        # set to None if suffix (nullable) is None
        # and model_fields_set contains the field
        if self.suffix is None and "suffix" in self.model_fields_set:
            _dict['suffix'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RecoverCassandraParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "advancedConfigs": [KeyValuePair.from_dict(_item) for _item in obj["advancedConfigs"]] if obj.get("advancedConfigs") is not None else None,
            "bandwidthMBPS": obj.get("bandwidthMBPS"),
            "concurrency": obj.get("concurrency"),
            "overwrite": obj.get("overwrite"),
            "recoverTo": obj.get("recoverTo"),
            "warnings": obj.get("warnings"),
            "isLiveTableRestore": obj.get("isLiveTableRestore"),
            "isSystemKeyspaceRestore": obj.get("isSystemKeyspaceRestore"),
            "logRestoreDirectory": obj.get("logRestoreDirectory"),
            "recoverPrivileges": obj.get("recoverPrivileges"),
            "restartAtUsecs": obj.get("restartAtUsecs"),
            "restartCommand": obj.get("restartCommand"),
            "restartImmediately": obj.get("restartImmediately"),
            "restartServices": obj.get("restartServices"),
            "restartServicesTaskId": obj.get("restartServicesTaskId"),
            "selectedDataCenters": obj.get("selectedDataCenters"),
            "snapshots": [RecoverCassandraSnapshotParams.from_dict(_item) for _item in obj["snapshots"]] if obj.get("snapshots") is not None else None,
            "stagingDirectoryList": obj.get("stagingDirectoryList"),
            "suffix": obj.get("suffix")
        })
        return _obj


