# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.cluster.models.aag_info import AAGInfo
from cohesity_sdk.cluster.models.archival_target_summary_info import ArchivalTargetSummaryInfo
from cohesity_sdk.cluster.models.common_recover_sql_app_target_params import CommonRecoverSqlAppTargetParams
from cohesity_sdk.cluster.models.host_information import HostInformation
from cohesity_sdk.cluster.models.object_summary import ObjectSummary
from typing import Set
from typing_extensions import Self

class RecoverSqlAppParams(BaseModel):
    """
    Specifies the parameters to recover Sql databases.
    """ # noqa: E501
    archival_target_info: Optional[ArchivalTargetSummaryInfo] = Field(default=None, alias="archivalTargetInfo")
    bytes_restored: Optional[StrictInt] = Field(default=None, description="Specify the total bytes restored.", alias="bytesRestored")
    end_time_usecs: Optional[StrictInt] = Field(default=None, description="Specifies the end time of the Recovery in Unix timestamp epoch in microseconds. This field will be populated only after Recovery is finished.", alias="endTimeUsecs")
    messages: Optional[List[StrictStr]] = Field(default=None, description="Specify error messages about the object.")
    object_info: Optional[ObjectSummary] = Field(default=None, alias="objectInfo")
    point_in_time_usecs: Optional[StrictInt] = Field(default=None, description="Specifies the timestamp (in microseconds. from epoch) for recovering to a point-in-time in the past.", alias="pointInTimeUsecs")
    progress_task_id: Optional[StrictStr] = Field(default=None, description="Progress monitor task id for Recovery of VM.", alias="progressTaskId")
    protection_group_id: Optional[StrictStr] = Field(default=None, description="Specifies the protection group id of the object snapshot.", alias="protectionGroupId")
    protection_group_name: Optional[StrictStr] = Field(default=None, description="Specifies the protection group name of the object snapshot.", alias="protectionGroupName")
    recover_from_standby: Optional[StrictBool] = Field(default=None, description="Specifies that user wants to perform standby restore if it is enabled for this object.", alias="recoverFromStandby")
    snapshot_creation_time_usecs: Optional[StrictInt] = Field(default=None, description="Specifies the time when the snapshot is created in Unix timestamp epoch in microseconds.", alias="snapshotCreationTimeUsecs")
    snapshot_id: StrictStr = Field(description="Specifies the snapshot id.", alias="snapshotId")
    snapshot_target_type: Optional[StrictStr] = Field(default=None, description="Specifies the snapshot target type.", alias="snapshotTargetType")
    start_time_usecs: Optional[StrictInt] = Field(default=None, description="Specifies the start time of the Recovery in Unix timestamp epoch in microseconds.", alias="startTimeUsecs")
    status: Optional[StrictStr] = Field(default=None, description="Status of the Recovery. 'Running' indicates that the Recovery is still running. 'Canceled' indicates that the Recovery has been cancelled. 'Canceling' indicates that the Recovery is in the process of being cancelled. 'Failed' indicates that the Recovery has failed. 'Succeeded' indicates that the Recovery has finished successfully. 'SucceededWithWarning' indicates that the Recovery finished successfully, but there were some warning messages. 'Skipped' indicates that the Recovery task was skipped.")
    storage_domain_id: Optional[StrictInt] = Field(default=None, description="Specifies the ID of the Storage Domain where this snapshot is stored.", alias="storageDomainId")
    aag_info: Optional[AAGInfo] = Field(default=None, alias="aagInfo")
    host_info: Optional[HostInformation] = Field(default=None, alias="hostInfo")
    is_encrypted: Optional[StrictBool] = Field(default=None, description="Specifies whether the database is TDE enabled.", alias="isEncrypted")
    sql_target_params: Optional[CommonRecoverSqlAppTargetParams] = Field(default=None, alias="sqlTargetParams")
    target_environment: StrictStr = Field(description="Specifies the environment of the recovery target. The corresponding params below must be filled out.", alias="targetEnvironment")
    __properties: ClassVar[List[str]] = ["archivalTargetInfo", "bytesRestored", "endTimeUsecs", "messages", "objectInfo", "pointInTimeUsecs", "progressTaskId", "protectionGroupId", "protectionGroupName", "recoverFromStandby", "snapshotCreationTimeUsecs", "snapshotId", "snapshotTargetType", "startTimeUsecs", "status", "storageDomainId", "aagInfo", "hostInfo", "isEncrypted", "sqlTargetParams", "targetEnvironment"]

    @field_validator('snapshot_target_type')
    def snapshot_target_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Local', 'Archival', 'RpaasArchival', 'StorageArraySnapshot', 'Remote']):
            raise ValueError("must be one of enum values ('Local', 'Archival', 'RpaasArchival', 'StorageArraySnapshot', 'Remote')")
        return value

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Accepted', 'Running', 'Canceled', 'Canceling', 'Failed', 'Missed', 'Succeeded', 'SucceededWithWarning', 'OnHold', 'Finalizing', 'Skipped']):
            raise ValueError("must be one of enum values ('Accepted', 'Running', 'Canceled', 'Canceling', 'Failed', 'Missed', 'Succeeded', 'SucceededWithWarning', 'OnHold', 'Finalizing', 'Skipped')")
        return value

    @field_validator('target_environment')
    def target_environment_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['kSQL']):
            raise ValueError("must be one of enum values ('kSQL')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RecoverSqlAppParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "bytes_restored",
            "end_time_usecs",
            "messages",
            "progress_task_id",
            "snapshot_creation_time_usecs",
            "snapshot_target_type",
            "start_time_usecs",
            "status",
            "storage_domain_id",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of archival_target_info
        if self.archival_target_info:
            _dict['archivalTargetInfo'] = self.archival_target_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of object_info
        if self.object_info:
            _dict['objectInfo'] = self.object_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of aag_info
        if self.aag_info:
            _dict['aagInfo'] = self.aag_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of host_info
        if self.host_info:
            _dict['hostInfo'] = self.host_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of sql_target_params
        if self.sql_target_params:
            _dict['sqlTargetParams'] = self.sql_target_params.to_dict()
        # set to None if bytes_restored (nullable) is None
        # and model_fields_set contains the field
        if self.bytes_restored is None and "bytes_restored" in self.model_fields_set:
            _dict['bytesRestored'] = None

        # set to None if end_time_usecs (nullable) is None
        # and model_fields_set contains the field
        if self.end_time_usecs is None and "end_time_usecs" in self.model_fields_set:
            _dict['endTimeUsecs'] = None

        # set to None if messages (nullable) is None
        # and model_fields_set contains the field
        if self.messages is None and "messages" in self.model_fields_set:
            _dict['messages'] = None

        # set to None if point_in_time_usecs (nullable) is None
        # and model_fields_set contains the field
        if self.point_in_time_usecs is None and "point_in_time_usecs" in self.model_fields_set:
            _dict['pointInTimeUsecs'] = None

        # set to None if progress_task_id (nullable) is None
        # and model_fields_set contains the field
        if self.progress_task_id is None and "progress_task_id" in self.model_fields_set:
            _dict['progressTaskId'] = None

        # set to None if protection_group_id (nullable) is None
        # and model_fields_set contains the field
        if self.protection_group_id is None and "protection_group_id" in self.model_fields_set:
            _dict['protectionGroupId'] = None

        # set to None if protection_group_name (nullable) is None
        # and model_fields_set contains the field
        if self.protection_group_name is None and "protection_group_name" in self.model_fields_set:
            _dict['protectionGroupName'] = None

        # set to None if recover_from_standby (nullable) is None
        # and model_fields_set contains the field
        if self.recover_from_standby is None and "recover_from_standby" in self.model_fields_set:
            _dict['recoverFromStandby'] = None

        # set to None if snapshot_creation_time_usecs (nullable) is None
        # and model_fields_set contains the field
        if self.snapshot_creation_time_usecs is None and "snapshot_creation_time_usecs" in self.model_fields_set:
            _dict['snapshotCreationTimeUsecs'] = None

        # set to None if snapshot_target_type (nullable) is None
        # and model_fields_set contains the field
        if self.snapshot_target_type is None and "snapshot_target_type" in self.model_fields_set:
            _dict['snapshotTargetType'] = None

        # set to None if start_time_usecs (nullable) is None
        # and model_fields_set contains the field
        if self.start_time_usecs is None and "start_time_usecs" in self.model_fields_set:
            _dict['startTimeUsecs'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['status'] = None

        # set to None if storage_domain_id (nullable) is None
        # and model_fields_set contains the field
        if self.storage_domain_id is None and "storage_domain_id" in self.model_fields_set:
            _dict['storageDomainId'] = None

        # set to None if is_encrypted (nullable) is None
        # and model_fields_set contains the field
        if self.is_encrypted is None and "is_encrypted" in self.model_fields_set:
            _dict['isEncrypted'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RecoverSqlAppParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "archivalTargetInfo": ArchivalTargetSummaryInfo.from_dict(obj["archivalTargetInfo"]) if obj.get("archivalTargetInfo") is not None else None,
            "bytesRestored": obj.get("bytesRestored"),
            "endTimeUsecs": obj.get("endTimeUsecs"),
            "messages": obj.get("messages"),
            "objectInfo": ObjectSummary.from_dict(obj["objectInfo"]) if obj.get("objectInfo") is not None else None,
            "pointInTimeUsecs": obj.get("pointInTimeUsecs"),
            "progressTaskId": obj.get("progressTaskId"),
            "protectionGroupId": obj.get("protectionGroupId"),
            "protectionGroupName": obj.get("protectionGroupName"),
            "recoverFromStandby": obj.get("recoverFromStandby"),
            "snapshotCreationTimeUsecs": obj.get("snapshotCreationTimeUsecs"),
            "snapshotId": obj.get("snapshotId"),
            "snapshotTargetType": obj.get("snapshotTargetType"),
            "startTimeUsecs": obj.get("startTimeUsecs"),
            "status": obj.get("status"),
            "storageDomainId": obj.get("storageDomainId"),
            "aagInfo": AAGInfo.from_dict(obj["aagInfo"]) if obj.get("aagInfo") is not None else None,
            "hostInfo": HostInformation.from_dict(obj["hostInfo"]) if obj.get("hostInfo") is not None else None,
            "isEncrypted": obj.get("isEncrypted"),
            "sqlTargetParams": CommonRecoverSqlAppTargetParams.from_dict(obj["sqlTargetParams"]) if obj.get("sqlTargetParams") is not None else None,
            "targetEnvironment": obj.get("targetEnvironment")
        })
        return _obj


