# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.cluster.models.common_s3_comp_external_target_params import CommonS3CompExternalTargetParams
from cohesity_sdk.cluster.models.tiering_aws_external_target_params import TieringAwsExternalTargetParams
from cohesity_sdk.cluster.models.tiering_azure_external_target_params import TieringAzureExternalTargetParams
from cohesity_sdk.cluster.models.tiering_gcp_external_target_params import TieringGcpExternalTargetParams
from cohesity_sdk.cluster.models.tiering_oracle_external_target_params import TieringOracleExternalTargetParams
from typing import Set
from typing_extensions import Self

class TieringExternalTargetParams(BaseModel):
    """
    Specifies the parameters which are specific to Tiering purpose type External Targets.
    """ # noqa: E501
    encryption_level: Optional[StrictStr] = Field(description="Specifies the type of encryption for the Setting.", alias="encryptionLevel")
    storage_type: Optional[StrictStr] = Field(description="Specifies the Storage type of the External Target.", alias="storageType")
    aws_params: Optional[TieringAwsExternalTargetParams] = Field(default=None, alias="awsParams")
    azure_params: Optional[TieringAzureExternalTargetParams] = Field(default=None, alias="azureParams")
    gcp_params: Optional[TieringGcpExternalTargetParams] = Field(default=None, alias="gcpParams")
    oracle_params: Optional[TieringOracleExternalTargetParams] = Field(default=None, alias="oracleParams")
    s3_comp_params: Optional[CommonS3CompExternalTargetParams] = Field(default=None, alias="s3CompParams")
    __properties: ClassVar[List[str]] = ["encryptionLevel", "storageType", "awsParams", "azureParams", "gcpParams", "oracleParams", "s3CompParams"]

    @field_validator('encryption_level')
    def encryption_level_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['None', 'Weak', 'Strong']):
            raise ValueError("must be one of enum values ('None', 'Weak', 'Strong')")
        return value

    @field_validator('storage_type')
    def storage_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Azure', 'Google', 'AWS', 'Oracle', 'S3Compatible']):
            raise ValueError("must be one of enum values ('Azure', 'Google', 'AWS', 'Oracle', 'S3Compatible')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TieringExternalTargetParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of aws_params
        if self.aws_params:
            _dict['awsParams'] = self.aws_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of azure_params
        if self.azure_params:
            _dict['azureParams'] = self.azure_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of gcp_params
        if self.gcp_params:
            _dict['gcpParams'] = self.gcp_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of oracle_params
        if self.oracle_params:
            _dict['oracleParams'] = self.oracle_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of s3_comp_params
        if self.s3_comp_params:
            _dict['s3CompParams'] = self.s3_comp_params.to_dict()
        # set to None if encryption_level (nullable) is None
        # and model_fields_set contains the field
        if self.encryption_level is None and "encryption_level" in self.model_fields_set:
            _dict['encryptionLevel'] = None

        # set to None if storage_type (nullable) is None
        # and model_fields_set contains the field
        if self.storage_type is None and "storage_type" in self.model_fields_set:
            _dict['storageType'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TieringExternalTargetParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "encryptionLevel": obj.get("encryptionLevel"),
            "storageType": obj.get("storageType"),
            "awsParams": TieringAwsExternalTargetParams.from_dict(obj["awsParams"]) if obj.get("awsParams") is not None else None,
            "azureParams": TieringAzureExternalTargetParams.from_dict(obj["azureParams"]) if obj.get("azureParams") is not None else None,
            "gcpParams": TieringGcpExternalTargetParams.from_dict(obj["gcpParams"]) if obj.get("gcpParams") is not None else None,
            "oracleParams": TieringOracleExternalTargetParams.from_dict(obj["oracleParams"]) if obj.get("oracleParams") is not None else None,
            "s3CompParams": CommonS3CompExternalTargetParams.from_dict(obj["s3CompParams"]) if obj.get("s3CompParams") is not None else None
        })
        return _obj


