# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from cohesity_sdk.cluster.models.key_value_pair import KeyValuePair
from cohesity_sdk.cluster.models.uda_object_protection_object_params import UdaObjectProtectionObjectParams
from typing import Set
from typing_extensions import Self

class UdaObjectProtectionParams(BaseModel):
    """
    Specifies the parameters that are specific to Universal Data Adapter Object Protection.
    """ # noqa: E501
    backup_job_arguments: Optional[List[KeyValuePair]] = Field(default=None, description="Specifies the map of custom arguments to be supplied to the various backup scripts.", alias="backupJobArguments")
    concurrency: Optional[StrictInt] = Field(default=1, description="Specifies the maximum number of concurrent IO Streams thatwill be created to exchange data with the cluster. If not specified, the default value is taken as 1.")
    full_backup_args: Optional[StrictStr] = Field(default=None, description="Specifies the custom arguments to be supplied to the full backup script when a full backup is enabled in the policy. This field is deprecated. Use backupJobArguments instead.", alias="fullBackupArgs")
    has_entity_support: Optional[StrictBool] = Field(default=None, description="Specifies whether this Protection Group is created from a source having entity support.", alias="hasEntitySupport")
    incr_backup_args: Optional[StrictStr] = Field(default=None, description="Specifies the custom arguments to be supplied to the incremental backup script when an incremental backup is enabled in the policy. This field is deprecated. Use backupJobArguments instead.", alias="incrBackupArgs")
    log_backup_args: Optional[StrictStr] = Field(default=None, description="Specifies the custom arguments to be supplied to the log backup script when a log backup is enabled in the policy. This field is deprecated. Use backupJobArguments instead.", alias="logBackupArgs")
    mounts: Optional[StrictInt] = Field(default=1, description="Specifies the maximum number of view mounts per host. If not specified, the default value is taken as 1.")
    objects: Annotated[List[UdaObjectProtectionObjectParams], Field(min_length=1)] = Field(description="Specifies the objects to be included in the Object Protection.")
    __properties: ClassVar[List[str]] = ["backupJobArguments", "concurrency", "fullBackupArgs", "hasEntitySupport", "incrBackupArgs", "logBackupArgs", "mounts", "objects"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UdaObjectProtectionParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "has_entity_support",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in backup_job_arguments (list)
        _items = []
        if self.backup_job_arguments:
            for _item_backup_job_arguments in self.backup_job_arguments:
                if _item_backup_job_arguments:
                    _items.append(_item_backup_job_arguments.to_dict())
            _dict['backupJobArguments'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in objects (list)
        _items = []
        if self.objects:
            for _item_objects in self.objects:
                if _item_objects:
                    _items.append(_item_objects.to_dict())
            _dict['objects'] = _items
        # set to None if backup_job_arguments (nullable) is None
        # and model_fields_set contains the field
        if self.backup_job_arguments is None and "backup_job_arguments" in self.model_fields_set:
            _dict['backupJobArguments'] = None

        # set to None if concurrency (nullable) is None
        # and model_fields_set contains the field
        if self.concurrency is None and "concurrency" in self.model_fields_set:
            _dict['concurrency'] = None

        # set to None if full_backup_args (nullable) is None
        # and model_fields_set contains the field
        if self.full_backup_args is None and "full_backup_args" in self.model_fields_set:
            _dict['fullBackupArgs'] = None

        # set to None if has_entity_support (nullable) is None
        # and model_fields_set contains the field
        if self.has_entity_support is None and "has_entity_support" in self.model_fields_set:
            _dict['hasEntitySupport'] = None

        # set to None if incr_backup_args (nullable) is None
        # and model_fields_set contains the field
        if self.incr_backup_args is None and "incr_backup_args" in self.model_fields_set:
            _dict['incrBackupArgs'] = None

        # set to None if log_backup_args (nullable) is None
        # and model_fields_set contains the field
        if self.log_backup_args is None and "log_backup_args" in self.model_fields_set:
            _dict['logBackupArgs'] = None

        # set to None if mounts (nullable) is None
        # and model_fields_set contains the field
        if self.mounts is None and "mounts" in self.model_fields_set:
            _dict['mounts'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UdaObjectProtectionParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "backupJobArguments": [KeyValuePair.from_dict(_item) for _item in obj["backupJobArguments"]] if obj.get("backupJobArguments") is not None else None,
            "concurrency": obj.get("concurrency") if obj.get("concurrency") is not None else 1,
            "fullBackupArgs": obj.get("fullBackupArgs"),
            "hasEntitySupport": obj.get("hasEntitySupport"),
            "incrBackupArgs": obj.get("incrBackupArgs"),
            "logBackupArgs": obj.get("logBackupArgs"),
            "mounts": obj.get("mounts") if obj.get("mounts") is not None else 1,
            "objects": [UdaObjectProtectionObjectParams.from_dict(_item) for _item in obj["objects"]] if obj.get("objects") is not None else None
        })
        return _obj


