# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from cohesity_sdk.helios.models.creation_info import CreationInfo
from cohesity_sdk.helios.models.retrieve_archive_task import RetrieveArchiveTask
from cohesity_sdk.helios.models.tenant import Tenant
from typing import Set
from typing_extensions import Self

class CommonRecoveryResponseParams(BaseModel):
    """
    Specifies the common response parameters to create a Recovery
    """ # noqa: E501
    can_tear_down: Optional[StrictBool] = Field(default=None, description="Specifies whether it's possible to tear down the objects created by the recovery.", alias="canTearDown")
    creation_info: Optional[CreationInfo] = Field(default=None, alias="creationInfo")
    end_time_usecs: Optional[StrictInt] = Field(default=None, description="Specifies the end time of the Recovery in Unix timestamp epoch in microseconds. This field will be populated only after Recovery is finished.", alias="endTimeUsecs")
    id: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, description="Specifies the id of the Recovery.")
    is_multi_stage_restore: Optional[StrictBool] = Field(default=None, description="Specifies whether the current recovery operation is a multi-stage restore operation. This is currently used by VMware recoveres for the migration/hot-standby use case.", alias="isMultiStageRestore")
    is_parent_recovery: Optional[StrictBool] = Field(default=None, description="Specifies whether the current recovery operation has created child recoveries. This is currently used in SQL recovery where multiple child recoveries can be tracked under a common/parent recovery.", alias="isParentRecovery")
    messages: Optional[List[StrictStr]] = Field(default=None, description="Specifies messages about the recovery.")
    name: Optional[StrictStr] = Field(default=None, description="Specifies the name of the Recovery.")
    parent_recovery_id: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, description="If current recovery is child recovery triggered by another parent recovery operation, then this field willt specify the id of the parent recovery.", alias="parentRecoveryId")
    permissions: Optional[List[Tenant]] = Field(default=None, description="Specifies the list of tenants that have permissions for this recovery.")
    progress_task_id: Optional[StrictStr] = Field(default=None, description="Progress monitor task id for Recovery.", alias="progressTaskId")
    recovery_action: Optional[StrictStr] = Field(default=None, description="Specifies the type of recover action.", alias="recoveryAction")
    retrieve_archive_tasks: Optional[List[RetrieveArchiveTask]] = Field(default=None, description="Specifies the list of persistent state of a retrieve of an archive task.", alias="retrieveArchiveTasks")
    snapshot_environment: Optional[StrictStr] = Field(default=None, description="Specifies the type of snapshot environment for which the Recovery was performed.", alias="snapshotEnvironment")
    start_time_usecs: Optional[StrictInt] = Field(default=None, description="Specifies the start time of the Recovery in Unix timestamp epoch in microseconds.", alias="startTimeUsecs")
    status: Optional[StrictStr] = Field(default=None, description="Status of the Recovery. 'Running' indicates that the Recovery is still running. 'Canceled' indicates that the Recovery has been cancelled. 'Canceling' indicates that the Recovery is in the process of being cancelled. 'Failed' indicates that the Recovery has failed. 'Succeeded' indicates that the Recovery has finished successfully. 'SucceededWithWarning' indicates that the Recovery finished successfully, but there were some warning messages. 'Skipped' indicates that the Recovery task was skipped.")
    tear_down_message: Optional[StrictStr] = Field(default=None, description="Specifies the error message about the tear down operation if it fails.", alias="tearDownMessage")
    tear_down_status: Optional[StrictStr] = Field(default=None, description="Specifies the status of the tear down operation. This is only set when the canTearDown is set to true. 'DestroyScheduled' indicates that the tear down is ready to schedule. 'Destroying' indicates that the tear down is still running. 'Destroyed' indicates that the tear down succeeded. 'DestroyError' indicates that the tear down failed.", alias="tearDownStatus")
    __properties: ClassVar[List[str]] = ["canTearDown", "creationInfo", "endTimeUsecs", "id", "isMultiStageRestore", "isParentRecovery", "messages", "name", "parentRecoveryId", "permissions", "progressTaskId", "recoveryAction", "retrieveArchiveTasks", "snapshotEnvironment", "startTimeUsecs", "status", "tearDownMessage", "tearDownStatus"]

    @field_validator('id')
    def id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^\d+:\d+:\d+$", value):
            raise ValueError(r"must validate the regular expression /^\d+:\d+:\d+$/")
        return value

    @field_validator('parent_recovery_id')
    def parent_recovery_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^\d+:\d+:\d+$", value):
            raise ValueError(r"must validate the regular expression /^\d+:\d+:\d+$/")
        return value

    @field_validator('recovery_action')
    def recovery_action_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['RecoverVMs', 'RecoverFiles', 'InstantVolumeMount', 'RecoverVmDisks', 'RecoverVApps', 'RecoverVAppTemplates', 'UptierSnapshot', 'RecoverRDS', 'RecoverAurora', 'RecoverS3Buckets', 'RecoverRDSPostgres', 'RecoverAzureSQL', 'RecoverApps', 'CloneApps', 'RecoverNasVolume', 'RecoverPhysicalVolumes', 'RecoverSystem', 'RecoverExchangeDbs', 'CloneAppView', 'RecoverSanVolumes', 'RecoverSanGroup', 'RecoverMailbox', 'RecoverOneDrive', 'RecoverSharePoint', 'RecoverPublicFolders', 'RecoverMsGroup', 'RecoverMsTeam', 'ConvertToPst', 'DownloadChats', 'RecoverNamespaces', 'RecoverObjects', 'RecoverSfdcObjects', 'RecoverSfdcOrg', 'RecoverSfdcRecords', 'DownloadFilesAndFolders', 'CloneVMs', 'CloneView', 'CloneRefreshApp', 'CloneVMsToView', 'ConvertAndDeployVMs', 'DeployVMs']):
            raise ValueError("must be one of enum values ('RecoverVMs', 'RecoverFiles', 'InstantVolumeMount', 'RecoverVmDisks', 'RecoverVApps', 'RecoverVAppTemplates', 'UptierSnapshot', 'RecoverRDS', 'RecoverAurora', 'RecoverS3Buckets', 'RecoverRDSPostgres', 'RecoverAzureSQL', 'RecoverApps', 'CloneApps', 'RecoverNasVolume', 'RecoverPhysicalVolumes', 'RecoverSystem', 'RecoverExchangeDbs', 'CloneAppView', 'RecoverSanVolumes', 'RecoverSanGroup', 'RecoverMailbox', 'RecoverOneDrive', 'RecoverSharePoint', 'RecoverPublicFolders', 'RecoverMsGroup', 'RecoverMsTeam', 'ConvertToPst', 'DownloadChats', 'RecoverNamespaces', 'RecoverObjects', 'RecoverSfdcObjects', 'RecoverSfdcOrg', 'RecoverSfdcRecords', 'DownloadFilesAndFolders', 'CloneVMs', 'CloneView', 'CloneRefreshApp', 'CloneVMsToView', 'ConvertAndDeployVMs', 'DeployVMs')")
        return value

    @field_validator('snapshot_environment')
    def snapshot_environment_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['kVMware', 'kHyperV', 'kAzure', 'kGCP', 'kKVM', 'kAcropolis', 'kAWS', 'kPhysical', 'kGPFS', 'kElastifile', 'kNetapp', 'kGenericNas', 'kIsilon', 'kFlashBlade', 'kPure', 'kIbmFlashSystem', 'kSQL', 'kExchange', 'kAD', 'kOracle', 'kView', 'kRemoteAdapter', 'kO365', 'kKubernetes', 'kCassandra', 'kMongoDB', 'kCouchbase', 'kHdfs', 'kHive', 'kHBase', 'kUDA', 'kSfdc']):
            raise ValueError("must be one of enum values ('kVMware', 'kHyperV', 'kAzure', 'kGCP', 'kKVM', 'kAcropolis', 'kAWS', 'kPhysical', 'kGPFS', 'kElastifile', 'kNetapp', 'kGenericNas', 'kIsilon', 'kFlashBlade', 'kPure', 'kIbmFlashSystem', 'kSQL', 'kExchange', 'kAD', 'kOracle', 'kView', 'kRemoteAdapter', 'kO365', 'kKubernetes', 'kCassandra', 'kMongoDB', 'kCouchbase', 'kHdfs', 'kHive', 'kHBase', 'kUDA', 'kSfdc')")
        return value

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Accepted', 'Running', 'Canceled', 'Canceling', 'Failed', 'Missed', 'Succeeded', 'SucceededWithWarning', 'OnHold', 'Finalizing', 'Skipped']):
            raise ValueError("must be one of enum values ('Accepted', 'Running', 'Canceled', 'Canceling', 'Failed', 'Missed', 'Succeeded', 'SucceededWithWarning', 'OnHold', 'Finalizing', 'Skipped')")
        return value

    @field_validator('tear_down_status')
    def tear_down_status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['DestroyScheduled', 'Destroying', 'Destroyed', 'DestroyError']):
            raise ValueError("must be one of enum values ('DestroyScheduled', 'Destroying', 'Destroyed', 'DestroyError')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CommonRecoveryResponseParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of creation_info
        if self.creation_info:
            _dict['creationInfo'] = self.creation_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in permissions (list)
        _items = []
        if self.permissions:
            for _item_permissions in self.permissions:
                if _item_permissions:
                    _items.append(_item_permissions.to_dict())
            _dict['permissions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in retrieve_archive_tasks (list)
        _items = []
        if self.retrieve_archive_tasks:
            for _item_retrieve_archive_tasks in self.retrieve_archive_tasks:
                if _item_retrieve_archive_tasks:
                    _items.append(_item_retrieve_archive_tasks.to_dict())
            _dict['retrieveArchiveTasks'] = _items
        # set to None if can_tear_down (nullable) is None
        # and model_fields_set contains the field
        if self.can_tear_down is None and "can_tear_down" in self.model_fields_set:
            _dict['canTearDown'] = None

        # set to None if end_time_usecs (nullable) is None
        # and model_fields_set contains the field
        if self.end_time_usecs is None and "end_time_usecs" in self.model_fields_set:
            _dict['endTimeUsecs'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if is_multi_stage_restore (nullable) is None
        # and model_fields_set contains the field
        if self.is_multi_stage_restore is None and "is_multi_stage_restore" in self.model_fields_set:
            _dict['isMultiStageRestore'] = None

        # set to None if is_parent_recovery (nullable) is None
        # and model_fields_set contains the field
        if self.is_parent_recovery is None and "is_parent_recovery" in self.model_fields_set:
            _dict['isParentRecovery'] = None

        # set to None if messages (nullable) is None
        # and model_fields_set contains the field
        if self.messages is None and "messages" in self.model_fields_set:
            _dict['messages'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['name'] = None

        # set to None if parent_recovery_id (nullable) is None
        # and model_fields_set contains the field
        if self.parent_recovery_id is None and "parent_recovery_id" in self.model_fields_set:
            _dict['parentRecoveryId'] = None

        # set to None if permissions (nullable) is None
        # and model_fields_set contains the field
        if self.permissions is None and "permissions" in self.model_fields_set:
            _dict['permissions'] = None

        # set to None if progress_task_id (nullable) is None
        # and model_fields_set contains the field
        if self.progress_task_id is None and "progress_task_id" in self.model_fields_set:
            _dict['progressTaskId'] = None

        # set to None if retrieve_archive_tasks (nullable) is None
        # and model_fields_set contains the field
        if self.retrieve_archive_tasks is None and "retrieve_archive_tasks" in self.model_fields_set:
            _dict['retrieveArchiveTasks'] = None

        # set to None if start_time_usecs (nullable) is None
        # and model_fields_set contains the field
        if self.start_time_usecs is None and "start_time_usecs" in self.model_fields_set:
            _dict['startTimeUsecs'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['status'] = None

        # set to None if tear_down_message (nullable) is None
        # and model_fields_set contains the field
        if self.tear_down_message is None and "tear_down_message" in self.model_fields_set:
            _dict['tearDownMessage'] = None

        # set to None if tear_down_status (nullable) is None
        # and model_fields_set contains the field
        if self.tear_down_status is None and "tear_down_status" in self.model_fields_set:
            _dict['tearDownStatus'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CommonRecoveryResponseParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "canTearDown": obj.get("canTearDown"),
            "creationInfo": CreationInfo.from_dict(obj["creationInfo"]) if obj.get("creationInfo") is not None else None,
            "endTimeUsecs": obj.get("endTimeUsecs"),
            "id": obj.get("id"),
            "isMultiStageRestore": obj.get("isMultiStageRestore"),
            "isParentRecovery": obj.get("isParentRecovery"),
            "messages": obj.get("messages"),
            "name": obj.get("name"),
            "parentRecoveryId": obj.get("parentRecoveryId"),
            "permissions": [Tenant.from_dict(_item) for _item in obj["permissions"]] if obj.get("permissions") is not None else None,
            "progressTaskId": obj.get("progressTaskId"),
            "recoveryAction": obj.get("recoveryAction"),
            "retrieveArchiveTasks": [RetrieveArchiveTask.from_dict(_item) for _item in obj["retrieveArchiveTasks"]] if obj.get("retrieveArchiveTasks") is not None else None,
            "snapshotEnvironment": obj.get("snapshotEnvironment"),
            "startTimeUsecs": obj.get("startTimeUsecs"),
            "status": obj.get("status"),
            "tearDownMessage": obj.get("tearDownMessage"),
            "tearDownStatus": obj.get("tearDownStatus")
        })
        return _obj


