# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.helios.models.aws_kms_configuration_response import AwsKmsConfigurationResponse
from cohesity_sdk.helios.models.kmip_kms_configuration_response import KmipKmsConfigurationResponse
from typing import Set
from typing_extensions import Self

class KmsConfiguration(BaseModel):
    """
     Key management system(KMS) configurations.
    """ # noqa: E501
    aws_kms_params: Optional[AwsKmsConfigurationResponse] = Field(default=None, alias="awsKmsParams")
    external_target_ids: Optional[List[StrictInt]] = Field(default=None, description="Ids of external targets used to assign the KMS for encryption. Once an external KMS (AWS KMS or KIMP KMS) is assigned to an external target, it cannot be changed.", alias="externalTargetIds")
    kmip_kms_params: Optional[KmipKmsConfigurationResponse] = Field(default=None, alias="kmipKmsParams")
    name: Optional[StrictStr] = Field(default=None, description="Name of the KMS.")
    ownership_context: Optional[StrictStr] = Field(default=None, description="Describes the consumption of the KMS key whether it is used for local or FortKnox.", alias="ownershipContext")
    storage_domain_ids: Optional[List[StrictInt]] = Field(default=None, description="Ids of storage domains used to assign the KMS for encryption. Once an external KMS (AWS KMS or KIMP KMS) is assigned to a storage domain, it cannot be changed.", alias="storageDomainIds")
    type: Optional[StrictStr] = Field(default=None, description="Type of KMS. 'InternalKms' indicates the internal cluster KMS. 'AwsKms' indicates AWS KMS. 'KmipKms' indicates any KMIP compliant KMS.")
    usage_type: Optional[StrictStr] = Field(default=None, description="Specifies the usage type of the kms config. 'kArchival' indicates this is used for regular archival. 'kRpaasArchival' indicates this is used for RPaaS only.", alias="usageType")
    id: Optional[StrictInt] = Field(default=None, description="Id of KMS.")
    state: Optional[StrictStr] = Field(default=None, description="Specifies the state of KMS. 'Active' indicates that KMS is reachable from cluster. 'InActive' indicates that KMS is not reachable from cluster. 'MarkedForRemoval' indicates that KMS is marked for removal and the removal process is in progress.")
    __properties: ClassVar[List[str]] = ["awsKmsParams", "externalTargetIds", "kmipKmsParams", "name", "ownershipContext", "storageDomainIds", "type", "usageType", "id", "state"]

    @field_validator('ownership_context')
    def ownership_context_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Local', 'FortKnox']):
            raise ValueError("must be one of enum values ('Local', 'FortKnox')")
        return value

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['InternalKms', 'AwsKms', 'KmipKms']):
            raise ValueError("must be one of enum values ('InternalKms', 'AwsKms', 'KmipKms')")
        return value

    @field_validator('usage_type')
    def usage_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['kArchival', 'kRpaasArchival']):
            raise ValueError("must be one of enum values ('kArchival', 'kRpaasArchival')")
        return value

    @field_validator('state')
    def state_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Active', 'InActive', 'MarkedForRemoval']):
            raise ValueError("must be one of enum values ('Active', 'InActive', 'MarkedForRemoval')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of KmsConfiguration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "id",
            "state",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of aws_kms_params
        if self.aws_kms_params:
            _dict['awsKmsParams'] = self.aws_kms_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of kmip_kms_params
        if self.kmip_kms_params:
            _dict['kmipKmsParams'] = self.kmip_kms_params.to_dict()
        # set to None if external_target_ids (nullable) is None
        # and model_fields_set contains the field
        if self.external_target_ids is None and "external_target_ids" in self.model_fields_set:
            _dict['externalTargetIds'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['name'] = None

        # set to None if ownership_context (nullable) is None
        # and model_fields_set contains the field
        if self.ownership_context is None and "ownership_context" in self.model_fields_set:
            _dict['ownershipContext'] = None

        # set to None if storage_domain_ids (nullable) is None
        # and model_fields_set contains the field
        if self.storage_domain_ids is None and "storage_domain_ids" in self.model_fields_set:
            _dict['storageDomainIds'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['type'] = None

        # set to None if usage_type (nullable) is None
        # and model_fields_set contains the field
        if self.usage_type is None and "usage_type" in self.model_fields_set:
            _dict['usageType'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if state (nullable) is None
        # and model_fields_set contains the field
        if self.state is None and "state" in self.model_fields_set:
            _dict['state'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of KmsConfiguration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "awsKmsParams": AwsKmsConfigurationResponse.from_dict(obj["awsKmsParams"]) if obj.get("awsKmsParams") is not None else None,
            "externalTargetIds": obj.get("externalTargetIds"),
            "kmipKmsParams": KmipKmsConfigurationResponse.from_dict(obj["kmipKmsParams"]) if obj.get("kmipKmsParams") is not None else None,
            "name": obj.get("name"),
            "ownershipContext": obj.get("ownershipContext"),
            "storageDomainIds": obj.get("storageDomainIds"),
            "type": obj.get("type"),
            "usageType": obj.get("usageType"),
            "id": obj.get("id"),
            "state": obj.get("state")
        })
        return _obj


