# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Set
from typing_extensions import Self

class NetappObjectParams(BaseModel):
    """
    Specifies the common parameters for Netapp objects.
    """ # noqa: E501
    supported_nas_mount_protocols: Optional[List[StrictStr]] = Field(default=None, description="Specifies a list of NAS mount protocols supported by this object.", alias="supportedNasMountProtocols")
    volume_extended_style: Optional[StrictStr] = Field(default=None, description="Specifies the extended style of a NetApp volume.", alias="volumeExtendedStyle")
    volume_type: Optional[StrictStr] = Field(default=None, description="Specifies the Netapp volume type.", alias="volumeType")
    __properties: ClassVar[List[str]] = ["supportedNasMountProtocols", "volumeExtendedStyle", "volumeType"]

    @field_validator('supported_nas_mount_protocols')
    def supported_nas_mount_protocols_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(['kNfs', 'kCifs', 'kIscsi', 'kFc', 'kFcache', 'kHttp', 'kNdmp', 'kManagement', 'kNvme']):
                raise ValueError("each list item must be one of ('kNfs', 'kCifs', 'kIscsi', 'kFc', 'kFcache', 'kHttp', 'kNdmp', 'kManagement', 'kNvme')")
        return value

    @field_validator('volume_extended_style')
    def volume_extended_style_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['kFlexVol', 'kFlexGroup']):
            raise ValueError("must be one of enum values ('kFlexVol', 'kFlexGroup')")
        return value

    @field_validator('volume_type')
    def volume_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['ReadWrite', 'LoadSharing', 'DataProtection', 'DataCache', 'Temp', 'UnkownType']):
            raise ValueError("must be one of enum values ('ReadWrite', 'LoadSharing', 'DataProtection', 'DataCache', 'Temp', 'UnkownType')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of NetappObjectParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if supported_nas_mount_protocols (nullable) is None
        # and model_fields_set contains the field
        if self.supported_nas_mount_protocols is None and "supported_nas_mount_protocols" in self.model_fields_set:
            _dict['supportedNasMountProtocols'] = None

        # set to None if volume_extended_style (nullable) is None
        # and model_fields_set contains the field
        if self.volume_extended_style is None and "volume_extended_style" in self.model_fields_set:
            _dict['volumeExtendedStyle'] = None

        # set to None if volume_type (nullable) is None
        # and model_fields_set contains the field
        if self.volume_type is None and "volume_type" in self.model_fields_set:
            _dict['volumeType'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of NetappObjectParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "supportedNasMountProtocols": obj.get("supportedNasMountProtocols"),
            "volumeExtendedStyle": obj.get("volumeExtendedStyle"),
            "volumeType": obj.get("volumeType")
        })
        return _obj


