# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.helios.models.common_download_file_and_folder_params import CommonDownloadFileAndFolderParams
from cohesity_sdk.helios.models.common_recover_object_snapshot_params import CommonRecoverObjectSnapshotParams
from cohesity_sdk.helios.models.mount_physical_volume_params import MountPhysicalVolumeParams
from cohesity_sdk.helios.models.recover_physical_file_and_folder_params import RecoverPhysicalFileAndFolderParams
from cohesity_sdk.helios.models.recover_physical_volume_params import RecoverPhysicalVolumeParams
from cohesity_sdk.helios.models.system_recovery_params import SystemRecoveryParams
from typing import Set
from typing_extensions import Self

class RecoverPhysicalParams(BaseModel):
    """
    Specifies the recovery options specific to Physical environment.
    """ # noqa: E501
    download_file_and_folder_params: Optional[CommonDownloadFileAndFolderParams] = Field(default=None, description="Specifies the parameters to download files and folders.", alias="downloadFileAndFolderParams")
    mount_volume_params: Optional[MountPhysicalVolumeParams] = Field(default=None, description="Specifies the parameters to mount Physical Volumes.", alias="mountVolumeParams")
    objects: Optional[List[CommonRecoverObjectSnapshotParams]] = Field(description="Specifies the list of Recover Object parameters. For recovering files, specifies the object contains the file to recover.")
    recover_file_and_folder_params: Optional[RecoverPhysicalFileAndFolderParams] = Field(default=None, description="Specifies the parameters to perform a file and folder recovery.", alias="recoverFileAndFolderParams")
    recover_volume_params: Optional[RecoverPhysicalVolumeParams] = Field(default=None, description="Specifies the parameters to recover Physical Volumes.", alias="recoverVolumeParams")
    recovery_action: StrictStr = Field(description="Specifies the type of recover action to be performed.", alias="recoveryAction")
    system_recovery_params: Optional[SystemRecoveryParams] = Field(default=None, description="Specifies the parameters to perform a system recovery.", alias="systemRecoveryParams")
    __properties: ClassVar[List[str]] = ["downloadFileAndFolderParams", "mountVolumeParams", "objects", "recoverFileAndFolderParams", "recoverVolumeParams", "recoveryAction", "systemRecoveryParams"]

    @field_validator('recovery_action')
    def recovery_action_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['RecoverPhysicalVolumes', 'InstantVolumeMount', 'RecoverFiles', 'RecoverSystem']):
            raise ValueError("must be one of enum values ('RecoverPhysicalVolumes', 'InstantVolumeMount', 'RecoverFiles', 'RecoverSystem')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RecoverPhysicalParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of download_file_and_folder_params
        if self.download_file_and_folder_params:
            _dict['downloadFileAndFolderParams'] = self.download_file_and_folder_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of mount_volume_params
        if self.mount_volume_params:
            _dict['mountVolumeParams'] = self.mount_volume_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in objects (list)
        _items = []
        if self.objects:
            for _item_objects in self.objects:
                if _item_objects:
                    _items.append(_item_objects.to_dict())
            _dict['objects'] = _items
        # override the default output from pydantic by calling `to_dict()` of recover_file_and_folder_params
        if self.recover_file_and_folder_params:
            _dict['recoverFileAndFolderParams'] = self.recover_file_and_folder_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of recover_volume_params
        if self.recover_volume_params:
            _dict['recoverVolumeParams'] = self.recover_volume_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of system_recovery_params
        if self.system_recovery_params:
            _dict['systemRecoveryParams'] = self.system_recovery_params.to_dict()
        # set to None if download_file_and_folder_params (nullable) is None
        # and model_fields_set contains the field
        if self.download_file_and_folder_params is None and "download_file_and_folder_params" in self.model_fields_set:
            _dict['downloadFileAndFolderParams'] = None

        # set to None if mount_volume_params (nullable) is None
        # and model_fields_set contains the field
        if self.mount_volume_params is None and "mount_volume_params" in self.model_fields_set:
            _dict['mountVolumeParams'] = None

        # set to None if objects (nullable) is None
        # and model_fields_set contains the field
        if self.objects is None and "objects" in self.model_fields_set:
            _dict['objects'] = None

        # set to None if recover_file_and_folder_params (nullable) is None
        # and model_fields_set contains the field
        if self.recover_file_and_folder_params is None and "recover_file_and_folder_params" in self.model_fields_set:
            _dict['recoverFileAndFolderParams'] = None

        # set to None if recover_volume_params (nullable) is None
        # and model_fields_set contains the field
        if self.recover_volume_params is None and "recover_volume_params" in self.model_fields_set:
            _dict['recoverVolumeParams'] = None

        # set to None if system_recovery_params (nullable) is None
        # and model_fields_set contains the field
        if self.system_recovery_params is None and "system_recovery_params" in self.model_fields_set:
            _dict['systemRecoveryParams'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RecoverPhysicalParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "downloadFileAndFolderParams": CommonDownloadFileAndFolderParams.from_dict(obj["downloadFileAndFolderParams"]) if obj.get("downloadFileAndFolderParams") is not None else None,
            "mountVolumeParams": MountPhysicalVolumeParams.from_dict(obj["mountVolumeParams"]) if obj.get("mountVolumeParams") is not None else None,
            "objects": [CommonRecoverObjectSnapshotParams.from_dict(_item) for _item in obj["objects"]] if obj.get("objects") is not None else None,
            "recoverFileAndFolderParams": RecoverPhysicalFileAndFolderParams.from_dict(obj["recoverFileAndFolderParams"]) if obj.get("recoverFileAndFolderParams") is not None else None,
            "recoverVolumeParams": RecoverPhysicalVolumeParams.from_dict(obj["recoverVolumeParams"]) if obj.get("recoverVolumeParams") is not None else None,
            "recoveryAction": obj.get("recoveryAction"),
            "systemRecoveryParams": SystemRecoveryParams.from_dict(obj["systemRecoveryParams"]) if obj.get("systemRecoveryParams") is not None else None
        })
        return _obj


