# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.helios.models.cassandra_connection_params import CassandraConnectionParams
from cohesity_sdk.helios.models.hadoop_connection_params import HadoopConnectionParams
from cohesity_sdk.helios.models.mssql_connection_params import MssqlConnectionParams
from cohesity_sdk.helios.models.oracle_connection_params import OracleConnectionParams
from cohesity_sdk.helios.models.vmware_connection_params import VmwareConnectionParams
from typing import Set
from typing_extensions import Self

class SourceConnectionRequestParams(BaseModel):
    """
    Specifies the parameters to test connectivity with a source.
    """ # noqa: E501
    connection_id: Optional[StrictInt] = Field(default=None, description="Specifies the id of the connection from where this source is reachable. This should only be set for a source being registered by a tenant user.", alias="connectionId")
    environment: Optional[StrictStr] = Field(description="Specifies the environment type of the Protection Source.")
    cassandra_connection_params: Optional[CassandraConnectionParams] = Field(default=None, alias="cassandraConnectionParams")
    hbase_connection_params: Optional[HadoopConnectionParams] = Field(default=None, alias="hbaseConnectionParams")
    hdfs_connection_params: Optional[HadoopConnectionParams] = Field(default=None, alias="hdfsConnectionParams")
    hive_connection_params: Optional[HadoopConnectionParams] = Field(default=None, alias="hiveConnectionParams")
    mssql_connection_params: Optional[MssqlConnectionParams] = Field(default=None, alias="mssqlConnectionParams")
    oracle_connection_params: Optional[OracleConnectionParams] = Field(default=None, alias="oracleConnectionParams")
    vmware_connection_params: Optional[VmwareConnectionParams] = Field(default=None, alias="vmwareConnectionParams")
    __properties: ClassVar[List[str]] = ["connectionId", "environment", "cassandraConnectionParams", "hbaseConnectionParams", "hdfsConnectionParams", "hiveConnectionParams", "mssqlConnectionParams", "oracleConnectionParams", "vmwareConnectionParams"]

    @field_validator('environment')
    def environment_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['kCassandra', 'kHive', 'kHBase', 'kHdfs', 'kSQL', 'kOracle']):
            raise ValueError("must be one of enum values ('kCassandra', 'kHive', 'kHBase', 'kHdfs', 'kSQL', 'kOracle')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SourceConnectionRequestParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of cassandra_connection_params
        if self.cassandra_connection_params:
            _dict['cassandraConnectionParams'] = self.cassandra_connection_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of hbase_connection_params
        if self.hbase_connection_params:
            _dict['hbaseConnectionParams'] = self.hbase_connection_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of hdfs_connection_params
        if self.hdfs_connection_params:
            _dict['hdfsConnectionParams'] = self.hdfs_connection_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of hive_connection_params
        if self.hive_connection_params:
            _dict['hiveConnectionParams'] = self.hive_connection_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of mssql_connection_params
        if self.mssql_connection_params:
            _dict['mssqlConnectionParams'] = self.mssql_connection_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of oracle_connection_params
        if self.oracle_connection_params:
            _dict['oracleConnectionParams'] = self.oracle_connection_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of vmware_connection_params
        if self.vmware_connection_params:
            _dict['vmwareConnectionParams'] = self.vmware_connection_params.to_dict()
        # set to None if connection_id (nullable) is None
        # and model_fields_set contains the field
        if self.connection_id is None and "connection_id" in self.model_fields_set:
            _dict['connectionId'] = None

        # set to None if environment (nullable) is None
        # and model_fields_set contains the field
        if self.environment is None and "environment" in self.model_fields_set:
            _dict['environment'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SourceConnectionRequestParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "connectionId": obj.get("connectionId"),
            "environment": obj.get("environment"),
            "cassandraConnectionParams": CassandraConnectionParams.from_dict(obj["cassandraConnectionParams"]) if obj.get("cassandraConnectionParams") is not None else None,
            "hbaseConnectionParams": HadoopConnectionParams.from_dict(obj["hbaseConnectionParams"]) if obj.get("hbaseConnectionParams") is not None else None,
            "hdfsConnectionParams": HadoopConnectionParams.from_dict(obj["hdfsConnectionParams"]) if obj.get("hdfsConnectionParams") is not None else None,
            "hiveConnectionParams": HadoopConnectionParams.from_dict(obj["hiveConnectionParams"]) if obj.get("hiveConnectionParams") is not None else None,
            "mssqlConnectionParams": MssqlConnectionParams.from_dict(obj["mssqlConnectionParams"]) if obj.get("mssqlConnectionParams") is not None else None,
            "oracleConnectionParams": OracleConnectionParams.from_dict(obj["oracleConnectionParams"]) if obj.get("oracleConnectionParams") is not None else None,
            "vmwareConnectionParams": VmwareConnectionParams.from_dict(obj["vmwareConnectionParams"]) if obj.get("vmwareConnectionParams") is not None else None
        })
        return _obj


