# coding: utf-8

"""
    Cohesity REST API

    Cohesity API provides a RESTful interface to access the various data management operations on Cohesity cluster and Helios.

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool
from typing import Any, ClassVar, Dict, List, Optional
from cohesity_sdk.helios.models.disk_info import DiskInfo
from cohesity_sdk.helios.models.indexing_policy import IndexingPolicy
from cohesity_sdk.helios.models.pre_post_script_params import PrePostScriptParams
from cohesity_sdk.helios.models.vmware_object_protection_request import VmwareObjectProtectionRequest
from typing import Set
from typing_extensions import Self

class VmwareObjectProtectionRequestParams(BaseModel):
    """
    Specifies the parameters which are specific to VMware object protection.
    """ # noqa: E501
    app_consistent_snapshot: Optional[StrictBool] = Field(default=None, description="Specifies whether or not to quiesce apps and the file system in order to take app consistent snapshots.", alias="appConsistentSnapshot")
    enable_nbdssl_fallback: Optional[StrictBool] = Field(default=None, description="If this field is set to true and SAN transport backup fails, then backup will fallback to use NBDSSL transport. This field only applies if 'leverageSanTransport' is set to true.", alias="enableNBDSSLFallback")
    fallback_to_crash_consistent_snapshot: Optional[StrictBool] = Field(default=None, description="Specifies whether or not to fallback to a crash consistent snapshot in the event that an app consistent snapshot fails. This parameter defaults to true and only changes the behavior of the operation if 'appConsistentSnapshot' is set to 'true'.", alias="fallbackToCrashConsistentSnapshot")
    indexing_policy: Optional[IndexingPolicy] = Field(default=None, alias="indexingPolicy")
    leverage_san_transport: Optional[StrictBool] = Field(default=None, description="If this field is set to true, then the backup for the objects will be performed using dedicated storage area network (SAN) instead of LAN or managment network.", alias="leverageSanTransport")
    pre_post_script: Optional[PrePostScriptParams] = Field(default=None, alias="prePostScript")
    skip_physical_rdm_disks: Optional[StrictBool] = Field(default=None, description="Specifies whether or not to skip backing up physical RDM disks. Physical RDM disks cannot be backed up, so if you attempt to backup a VM with physical RDM disks and this value is set to 'false', then those VM backups will fail.", alias="skipPhysicalRDMDisks")
    global_exclude_disks: Optional[List[DiskInfo]] = Field(default=None, description="Specifies a list of disks to exclude from the backup.", alias="globalExcludeDisks")
    objects: List[VmwareObjectProtectionRequest] = Field(description="Specifies the objects to include in the backup.")
    __properties: ClassVar[List[str]] = ["appConsistentSnapshot", "enableNBDSSLFallback", "fallbackToCrashConsistentSnapshot", "indexingPolicy", "leverageSanTransport", "prePostScript", "skipPhysicalRDMDisks", "globalExcludeDisks", "objects"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of VmwareObjectProtectionRequestParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of indexing_policy
        if self.indexing_policy:
            _dict['indexingPolicy'] = self.indexing_policy.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pre_post_script
        if self.pre_post_script:
            _dict['prePostScript'] = self.pre_post_script.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in global_exclude_disks (list)
        _items = []
        if self.global_exclude_disks:
            for _item_global_exclude_disks in self.global_exclude_disks:
                if _item_global_exclude_disks:
                    _items.append(_item_global_exclude_disks.to_dict())
            _dict['globalExcludeDisks'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in objects (list)
        _items = []
        if self.objects:
            for _item_objects in self.objects:
                if _item_objects:
                    _items.append(_item_objects.to_dict())
            _dict['objects'] = _items
        # set to None if app_consistent_snapshot (nullable) is None
        # and model_fields_set contains the field
        if self.app_consistent_snapshot is None and "app_consistent_snapshot" in self.model_fields_set:
            _dict['appConsistentSnapshot'] = None

        # set to None if enable_nbdssl_fallback (nullable) is None
        # and model_fields_set contains the field
        if self.enable_nbdssl_fallback is None and "enable_nbdssl_fallback" in self.model_fields_set:
            _dict['enableNBDSSLFallback'] = None

        # set to None if fallback_to_crash_consistent_snapshot (nullable) is None
        # and model_fields_set contains the field
        if self.fallback_to_crash_consistent_snapshot is None and "fallback_to_crash_consistent_snapshot" in self.model_fields_set:
            _dict['fallbackToCrashConsistentSnapshot'] = None

        # set to None if leverage_san_transport (nullable) is None
        # and model_fields_set contains the field
        if self.leverage_san_transport is None and "leverage_san_transport" in self.model_fields_set:
            _dict['leverageSanTransport'] = None

        # set to None if skip_physical_rdm_disks (nullable) is None
        # and model_fields_set contains the field
        if self.skip_physical_rdm_disks is None and "skip_physical_rdm_disks" in self.model_fields_set:
            _dict['skipPhysicalRDMDisks'] = None

        # set to None if global_exclude_disks (nullable) is None
        # and model_fields_set contains the field
        if self.global_exclude_disks is None and "global_exclude_disks" in self.model_fields_set:
            _dict['globalExcludeDisks'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of VmwareObjectProtectionRequestParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "appConsistentSnapshot": obj.get("appConsistentSnapshot"),
            "enableNBDSSLFallback": obj.get("enableNBDSSLFallback"),
            "fallbackToCrashConsistentSnapshot": obj.get("fallbackToCrashConsistentSnapshot"),
            "indexingPolicy": IndexingPolicy.from_dict(obj["indexingPolicy"]) if obj.get("indexingPolicy") is not None else None,
            "leverageSanTransport": obj.get("leverageSanTransport"),
            "prePostScript": PrePostScriptParams.from_dict(obj["prePostScript"]) if obj.get("prePostScript") is not None else None,
            "skipPhysicalRDMDisks": obj.get("skipPhysicalRDMDisks"),
            "globalExcludeDisks": [DiskInfo.from_dict(_item) for _item in obj["globalExcludeDisks"]] if obj.get("globalExcludeDisks") is not None else None,
            "objects": [VmwareObjectProtectionRequest.from_dict(_item) for _item in obj["objects"]] if obj.get("objects") is not None else None
        })
        return _obj


